#if !defined  HAVE_ENDO_H__
#define       HAVE_ENDO_H__


#include "fxttypes.h"

static inline ulong next_endo(ulong x, ulong m)
// Return next number in endo order
// (endo := "Even Numbers Down, Odd (numbers up)")
// m := max digit
//  m:   seq
//  1:   1 0
//  2:   1 2 0
//  3:   1 3 2 0
//  4:   1 3 4 2 0
//  5:   1 3 5 4 2 0
//  6:   1 3 5 6 4 2 0
//  7:   1 3 5 7 6 4 2 0
//  8:   1 3 5 7 8 6 4 2 0
//  9:   1 3 5 7 9 8 6 4 2 0
// The routine computes one for the input zero.
{
    if ( x & 1 )  // x odd
    {
        x += 2;
        if ( x>m )  x = m - (m&1);  // == max even <= m
    }
    else  // x even
    {
        x -= 2;
        if ( x>m )  x = 1;  // == min odd
    }
    return x;
}
// -------------------------

static inline ulong prev_endo(ulong x, ulong m)
// Return previous number in endo order
// Inverse of next_endo()
//  m:   seq
//  1:   1 0
//  2:   2 1 0
//  3:   2 3 1 0
//  4:   2 4 3 1 0
//  5:   2 4 5 3 1 0
//  6:   2 4 6 5 3 1 0
//  7:   2 4 6 7 5 3 1 0
//  8:   2 4 6 8 7 5 3 1 0
//  9:   2 4 6 8 9 7 5 3 1 0
{
    if ( x & 1 )  // x odd
    {
        x -= 2;
        if ( x>m )  x = 0;  // == min even
    }
    else  // x even
    {
        x += 2;
        if ( x>m )  x = m - !(m&1);  // max odd <=m
    }
    return x;
}
// -------------------------

static inline ulong last_endo(ulong m)
{
    return  2 - ( m==1 );
}
// -------------------------


#endif  // !defined HAVE_ENDO_H__
