#if !defined HAVE_MIXEDRADIX_GRAY2_H__
#define      HAVE_MIXEDRADIX_GRAY2_H__

#include "fxttypes.h"
#include "comb/printmixedradix.h"
#include "comb/mixedradix2num.h"


class mixedradix_gray2
// Gray code for mixed radix numbers.
// CAT algorithm.
{
public:
    ulong *a_;  // mixed radix digits
    ulong *m1_; // radices (minus one)
    ulong *i_;  // direction
    ulong n_;   // n_ digits
    ulong j_;   // position of last change
    int dm_;    // direction of last move

public:
    mixedradix_gray2(const ulong *m, ulong n, ulong mm=0)
    {
        n_ = n;
        a_ = new ulong[n_+1];
        a_[n] = -1UL;  // sentinel
        i_ = new ulong[n_+1];
        i_[n_] = 0;    // sentinel
        m1_ = new ulong[n_+1];

        if ( 0==mm )  for (ulong k=0; k<=n_; ++k)  m1_[k] = m[k] - 1;
        else          for (ulong k=0; k<=n_; ++k)  m1_[k] = mm - 1;

        first();
    }

    ~mixedradix_gray2()
    {
        delete [] i_;
        delete [] a_;
        delete [] m1_;
    }

    const ulong *data()  const { return a_; }

    void first()
    {
        for (ulong k=0; k<n_; ++k)  a_[k] = 0;
        for (ulong k=0; k<n_; ++k)  i_[k] = +1;
        j_ = n_+1;
        dm_ = 0;
    }

    void last()
    {
        for (ulong k=0; k<n_; ++k)  a_[k] = 0;
        for (ulong k=0; k<n_; ++k)  i_[k] = -1UL;

        // find position of last even radix:
        ulong z = 0;
        for (ulong i=0; i<n_; ++i)  if ( m1_[i]&1 )  z = i;
        while ( z<n_ )  // last even .. end:
        {
            a_[z] = m1_[z];
            i_[z] = +1;
            ++z;
        }

        j_ = 0;
        dm_ = -1;
    }

    bool next()
    {
        ulong j = 0;
        ulong ij;
//        while ( j<n_ )
        while ( (ij=i_[j]) )  // can touch sentinel i[n]==0
        {
            ulong dj = a_[j] + ij;

            if ( dj>m1_[j] )  // =^= if ( (dj>m1_[j]) || ((long)dj<0) )
            {
                i_[j] = -ij;
            }
            else
            {
                a_[j] = dj;
                dm_ = ij;
                j_ = j;
                return true;
            }

            ++j;
        }
        return false;
    }

    bool prev()
    {
        ulong j = 0;
        ulong ij;
//        while ( j<n_ )
        while ( (ij=i_[j]) )  // can touch sentinel i[n]==0
        {
            ulong dj = a_[j] - ij;

            if ( dj>m1_[j] )  // =^= if ( (dj>m1_[j]) || ((long)dj<0) )
            {
                i_[j] = -ij;
            }
            else
            {
                a_[j] = dj;
                dm_ = ij;
                j_ = j;
                return true;
            }

            ++j;
        }
        return false;
    }

    ulong pos()  const  { return j_; }  // position of last change
    int dir()  const  { return dm_; }  // direction of last change

    void print(const char *bla, bool dfz=false)  const
    // If dfz is true then Dots are printed For Zeros.
    { ::print_mixedradix(bla, a_, n_, dfz); }

    ulong to_num(bool rq=false)  const
    // Return (integer) value of mixed radix number.
    // If rq is true then the reversed digits are used.
    { return ::mixedradix2num(a_, m1_, n_, rq); }
};
// -------------------------


#endif  // !defined HAVE_MIXEDRADIX_GRAY2_H__
