#if !defined HAVE_MIXEDRADIX_MODULAR_GRAY_H__
#define      HAVE_MIXEDRADIX_MODULAR_GRAY_H__

#include "fxttypes.h"

#include "comb/printmixedradix.h"
#include "comb/mixedradix2num.h"


class mixedradix_modular_gray
// Modular Gray code for mixed radix numbers.
// Implementation following Knuth.
{
public:
    ulong *a_;  // digits
    ulong *m1_; // radix minus one ('nines')
    ulong *f_;  // focus pointer
    ulong *s_;  // direction
    ulong n_;   // number of digits
    ulong j_;   // position of last change

public:
    mixedradix_modular_gray(const ulong *m, ulong n, ulong mm=0)
        : n_(n)
    {
        a_ = new ulong[n_];
        m1_ = new ulong[n_];  // m1_[j] == m[j] - 1 in Knuth
        f_ = new ulong[n_+1]; // n_ + 1 elements
        s_ = new ulong[n_];

        // if mm given it is used as radix for all digits:
        if ( 0==mm )  for (ulong k=0; k<n_; ++k)  m1_[k] = m[k] - 1;
        else          for (ulong k=0; k<n_; ++k)  m1_[k] = mm - 1;

        first();
    }

    ~mixedradix_modular_gray()
    {
        delete [] a_;
        delete [] m1_;
        delete [] f_;
        delete [] s_;
    }

    const ulong *data()  const  { return a_; }


    void first()
    {
        for (ulong k=0; k<n_; ++k)  a_[k] = 0;
        for (ulong k=0; k<n_; ++k)  s_[k] = m1_[k];
        for (ulong k=0; k<=n_; ++k)  f_[k] = k;
        j_ = n_;
    }

    bool next()
    {
        j_ = f_[0];
        f_[0] = 0;

        if ( j_>=n_ )  { first();  return false; }

        ++a_[j_];  if ( a_[j_] > m1_[j_] )  a_[j_] = 0;

        ulong aj = a_[j_];
        if ( s_[j_]==aj )
        {
            --s_[j_];  if ( s_[j_] == -1UL )  s_[j_] = m1_[j_];

            f_[j_] = f_[j_+1];
            f_[j_+1] = j_ + 1;
        }

        return true;
    }

    ulong pos()  const  { return j_; }  // position of last change

    void print(const char *bla, bool dfz=false)  const
    // If dfz is true then Dots are printed For Zeros.
    { ::print_mixedradix(bla, a_, n_, dfz); }

    ulong to_num(bool rq=false)  const
    // Return (integer) value of mixed radix number.
    // If rq is true then the reversed digits are used.
    { return ::mixedradix2num(a_, m1_, n_, rq); }
};
// -------------------------


#endif  // !defined HAVE_MIXEDRADIX_MODULAR_GRAY_H__
