#if !defined HAVE_MIXEDRADIX_H__
#define      HAVE_MIXEDRADIX_H__

#include "fxttypes.h"
#include "comb/printmixedradix.h"
#include "comb/mixedradix2num.h"


class mixedradix
// Mixed radix counting.
{
public:
    ulong *a_;  // digits
    ulong *m1_; // radix (minus one) for each digit
    ulong n_;   // Number of digits
    ulong j_;   // position of last change

public:
    mixedradix(const ulong *m, ulong n, ulong mm=0)
    {
        n_ = n;
        a_ = new ulong[n_+1];
        a_[n_] = -1UL;  // sentinel
        m1_ = new ulong[n_];

        // if mm given it is used as radix for all digits:
        if ( 0==mm )  for (ulong k=0; k<n_; ++k)  m1_[k] = m[k] - 1;
        else          for (ulong k=0; k<n_; ++k)  m1_[k] = mm - 1;

        first();
    }

    ~mixedradix()
    {
        delete [] a_;
        delete [] m1_;
    }

    const ulong *data()  const  { return a_; }

    void first()
    {
        for (ulong k=0; k<n_; ++k)  a_[k] = 0;
        j_ = n_;
    }

    void last()
    {
        for (ulong k=0; k<n_; ++k)  a_[k] = m1_[k];
        j_ = n_;
    }


    bool next()  // increment
    {
        bool ret = false;
        ulong j = 0;
        while ( a_[j]==m1_[j] )  { a_[j]=0; ++j; }
        if ( j<n_ ) // only if no overflow
        {
            ++a_[j];
            ret = true;
        }
        j_ = j;
        return ret;
    }

    bool prev()  // decrement
    {
        bool ret = false;
        ulong j = 0;
        while ( a_[j]==0 )  { a_[j]=m1_[j]; ++j; }
        if ( j<n_ ) // only if no overflow
        {
            --a_[j];
            ret = true;
        }
        j_ = j;
        return ret;
    }

    ulong pos()  const  { return j_; }  // position of last change

    void print(const char *bla, bool dfz=false)  const
    // If dfz is true then Dots are printed For Zeros.
    { ::print_mixedradix(bla, a_, n_, dfz); }

    ulong to_num(bool rq=false)  const
    // Return (integer) value of mixed radix number.
    // If rq is true then the reversed digits are used.
    { return ::mixedradix2num(a_, m1_, n_, rq); }
};
// -------------------------


#endif  // !defined HAVE_MIXEDRADIX_H__
