#if !defined HAVE_PERMDERANGE_H__
#define      HAVE_PERMDERANGE_H__

#include "comb/permtrotter.h"
#include "comb/printperm.h"
#include "fxttypes.h"


class perm_derange
// Permutation in derangement order.
{
public:
    ulong n_;   // number of elements
    ulong *x_;  // current permutation
    ulong ctm_; // counter modulo n
    perm_trotter* pt;

public:
    perm_derange(ulong nn)
    {
        n_ = ( nn<4 ? 4 : nn );  // 2: trivial, 3: no solution exists,  4..: ok
        x_ = new ulong[n_];
        pt = new perm_trotter(n_-1);
        first();
    }

    ~perm_derange()
    {
        delete [] x_;
        delete pt;
    }

    void first()
    {
        ctm_ = 0;
        for (ulong k=0; k<n_; ++k)  x_[k] = k;
    }

    bool next()
    {
        ++ctm_;

        if ( ctm_>=n_ ) // every n steps: need next perm_trotter
        {
            ctm_ = 0;
            if ( 0==pt->next() )  return false;

            // copy in:
            const ulong *xx = pt->data();
            for (ulong k=0; k<n_-1; ++k)  x_[k] = xx[k];
            x_[n_-1] = n_-1;  // last element
        }
        else  // rotate
        {
            if ( ctm_==n_-1 )
            {
                rotl1(x_, n_);
            }
            else // last two swapped
            {
                rotr1(x_, n_);
                if ( ctm_==n_-2 )  rotr1(x_, n_);
            }
        }
        return true;
    }

    const ulong *data()  const  { return x_; }

    void print(const char *bla, bool dfz)  const
    // If dfz is true then Dots are printed For Zeros.
    { print_perm(bla, data(), n_, dfz); }

private:
    void rotl1(ulong *x_, ulong n_)
    // rotate array left by one position
    {
        ulong x0 = x_[0];
        for (ulong k=0; k<n_-1; ++k)  x_[k] = x_[k+1];
        x_[n_-1] = x0;
    }

    void rotr1(ulong *x_, ulong n_)
    // rotate array right by one position
    {
        ulong xl = x_[n_-1];
        for (ulong k=n_-1; k>0; --k)  x_[k] = x_[k-1];
        x_[0] = xl;
    }
};
// -------------------------


#endif  // !defined HAVE_PERMDERANGE_H__
