#if !defined HAVE_PERMLEX_H__
#define      HAVE_PERMLEX_H__

#include "aux0/swap.h"
#include "comb/printperm.h"
#include "fxttypes.h"


class perm_lex
// Permutations in lexicographic order
{
public:
    ulong *p_;  // permutation of {0, 1, ..., n-1}
    ulong n_;   // number of elements to permute

public:
    perm_lex(ulong n)
    {
        n_ = n;
        p_ = new ulong[n_+1];
        p_[n_] = ~0UL;  // sentinel: max ulong
        ++p_;
        first();
    }

    ~perm_lex() { --p_;  delete [] p_; }

    void first()
    {
        for (ulong i=0; i<n_; i++)  p_[i] = i;
    }

    const ulong *data()  const  { return p_; }

    bool next()
    // Based on code by Glenn Rhoads
    // (who ascribes the algorithm to Dijkstra)
    {
        const ulong n1 = n_ - 1;
        ulong i = n1;
        do  { --i; }  while ( p_[i] > p_[i+1] );

        if ( (long)i<0 )  return false;  // last sequence is falling seq.

        ulong j = n1;
        while ( p_[i] > p_[j] )  --j;

        swap2(p_[i], p_[j]);

        ulong r = n1;
        ulong s = i + 1;
        while ( r > s )
        {
            swap2(p_[r], p_[s]);
            --r;
            ++s;
        }

        return true;
    }

    void print(const char *bla, bool dfz)  const
    // If dfz is true then Dots are printed For Zeros.
    { print_perm(bla, data(), n_, dfz); }
};
// -------------------------



#endif  // !defined HAVE_PERMLEX_H__
