
#include "comb/combination-chase.h"
#include "aux0/binomial.h"

#include "fxttypes.h"

#include "jjassert.h"
#include "demo/nextarg.h"

#include "fxtiomanip.h"

//%  Generating all combinations in near-perfect minimal-change order (Chase's sequence).

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 7;
    NXARG(n, "(n choose k): n>=1");
    ulong k = 3;
    NXARG(k, "(n choose k):  1<=k<=n");
    bool rq = 0;
    NXARG(rq, "Option: print in reverse order");

    if ( (k>n) || ( k==0 ) || ( n==0 ) )  { return 1; }

    combination_chase C(n,k);

    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << setw(3) << ct << ":  ";

        const ulong *x = C.data();
        if ( rq )
        {
            for (ulong j=0; j<n; ++j)  if ( x[n-1-j] ) cout << " " << (n-1-j);
            cout << "    ";
            for (ulong j=0; j<n; ++j)  cout << " " << (x[n-1-j] ? '1' : '.');
        }
        else
        {
            for (ulong j=0; j<n; ++j)  if ( x[j] ) cout << " " << j;
            cout << "    ";
            for (ulong j=0; j<n; ++j)  cout << " " << (x[j] ? '1' : '.');
        }

        cout << endl;
#endif
        ++ct;
    }
    while ( C.next() );

    ulong bnk =  binomial(n, k);
    cout << "binomial(" << n << ", " << k << ")=" << bnk << endl;
    jjassert( bnk==ct );
    cout << endl;

    return 0;
}
// -------------------------

// for n in $(seq 1 10); do for k in $(seq 1 $n); do ./bin $n $k || break 2; done; done

/*
Timing:
% time ./bin 32 20
binomial(32, 20)=225792840
 ==> 64,512,240 objects per second

% time ./bin 32 12  is identical
*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/combination-chase-demo.cc"
/// End:

