

#include "comb/fact2perm.h"
// demo-include "comb/fact2perm.cc"
#include "perm/perminvert.h"  // make_inverse()
// demo-include "perm/perminvert.cc"
#include "comb/mixedradix-modular-gray.h"

#include "comb/permgray.h"
#include "comb/mixedradix.h"
#include "comb/mixedradix-gslex.h"
#include "comb/mixedradix-gray.h"
#include "comb/mixedradix-endo.h"
#include "comb/mixedradix-endo-gray.h"

#include "comb/printperm.h"
#include "comb/printmixedradix.h"

#include "jjassert.h"

#include "fxttypes.h"
#include "demo/nextarg.h"  // NXARG()
#include "fxtio.h"


//% Generate all permutations from mixed radix numbers.


int
main(int argc, char **argv)
{
    ulong n = 4;
    NXARG(n, "Number of elements to permute");
    bool rq = 0;
    NXARG(rq, "Whether to use reversed base with mixed radix counting");
    bool dfz = true;  // whether to print dots for zeros

    ulong n1 = n - 1;
    ulong *m = new ulong[n1];

    if ( rq )  for (ulong k=0; k<n1; ++k)  m[k] = k+2;  // factorial base
    else       for (ulong k=0; k<n1; ++k)  m[k] = n-k;  // reversed factorial base

    print_mixedradix(" Radix: ", m, n1);  cout << endl;

//    mixedradix_gray M(m, n1);
    mixedradix_modular_gray M(m, n1);  // default
//    mixedradix M(m, n1);
//    mixedradix_gslex M(m, n1);
//    mixedradix_endo M(m, n1);
//    mixedradix_endo_gray M(m, n1);

    const ulong *a = M.data();
    ulong *ia = new ulong[n1];
    ulong *t = new ulong[n];  // for testing

    ulong *p = new ulong[n];  // permutation
    ulong *ip = new ulong[n];  // inverse permutation
    ulong ct = 0;
    do
    {
        cout << " " << setw(3) << ct << ":";

        if ( rq )  rfact2perm(a, n, p);
        else       ffact2perm(a, n, p);

        print_mixedradix("    ", a, n1, dfz);
        print_perm("  ", p, n, dfz);

        cout << "    |  ";
        make_inverse(p, ip, n);
        if ( rq ) perm2rfact(ip, n, ia);
        else      perm2ffact(ip, n, ia);
        print_perm("    ", ip, n, dfz);
        print_mixedradix("  ", ia, n1, dfz);

        ++ct;
        cout << endl;


        // Testing that Xfact2perm() is inverse of perm2Xfact():
        if ( rq )  perm2rfact(p, n, t);
        else       perm2ffact(p, n, t);
//        print_perm("    ", t, n);  cout << endl;
        for (ulong j=0; j<n-1; ++j)  { jjassert(t[j]==a[j]); }
    }
    while ( M.next() );

    return 0;
}
// -------------------------



/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/fact2perm-demo.cc"
/// End:

