
#include "comb/mixedradix.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"

//% Mixed radix counting.


//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits")

    ALLOCA(ulong, r, n);
    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits");
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for second...last digit")
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-2] = atol(argv[k]);

    mixedradix M(r, n);
//    M.last();

    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();
        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " # = " << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 30 2  ## binary is worst case
 # = 1073741824
./bin 30 2  7.50s user 0.03s system 99% cpu 7.542 total
 ==> 143,165,576 objects per second

% time ./bin 16 4
 # = 4294967296
./bin 16 4  16.79s user 0.08s system 99% cpu 16.877 total
 ==> 255,805,080 objects per second

% time ./bin 10 8
 # = 1073741824
./bin 10 8  3.34s user 0.02s system 99% cpu 3.367 total
 ==> 321,479,588 objects per second

% time ./bin 8 16
 # = 4294967296
./bin 8 16  14.72s user 0.07s system 99% cpu 14.798 total
 ==> 291,777,669 objects per second

*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-demo.cc"
/// End:

