
#include "comb/mixedradix-endo.h"
#include "comb/endo.h"

#include "fxttypes.h"
#include "fxtio.h"
#include "jjassert.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"

//% Mixed radix counting: endo sequence
//% (endo := "Even Numbers Down, Odd (numbers up)")

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 2;
    NXARG(n,"Number of digits")

    ALLOCA(ulong, r, n);
    ulong rr = 6;
    NXARG(rr, "Base (radix) of digits");
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for second...last digit")
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-2] = atol(argv[k]);

    mixedradix_endo M(r, n);
//    M.last();

    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();
        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " # = " << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 30 2  ## binary is worst case
 # = 1073741824
./bin 30 2  9.85s user 0.04s system 99% cpu 9.902 total
 ==>  109,009,322 objects per second

% time ./bin 16 4
 # = 4294967296
./bin 16 4  31.67s user 0.15s system 99% cpu 31.832 total
 ==> 135,616,270  objects per second

% time ./bin 10 8
 # = 1073741824
./bin 10 8  7.21s user 0.04s system 99% cpu 7.258 total
 ==> 148,923,970  objects per second

% time ./bin 8 16
 # = 4294967296
./bin 8 16  26.12s user 0.16s system 99% cpu 26.307 total
 ==> 164,432,132  objects per second
*/



/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-endo-demo.cc"
/// End:

