
#include "comb/mixedradix-gray2.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"
#include "fxtiomanip.h"


//% Mixed radix Gray code, CAT algorithm.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits")

    ALLOCA(ulong, r, n);
    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits");
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for second...last digit")
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-2] = atol(argv[k]);

    mixedradix_gray2 M(r, n);
//    M.last();

    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();
        cout << setw(6) << M.pos() << "  " << (M.dir()<0?'-':'+') << 1;

//        cout << "    ";
//        for (ulong j=0; j<n; ++j)  cout << " " << ((long)(M.i_[j])<0?'-':'+');

        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " # = " << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 30 2  ## binary is worst case
 # = 1073741824
./bin 30 2  8.73s user 0.05s system 99% cpu 8.778 total
 ==> 122,994,481 objects per second

% time ./bin 16 4
 # = 4294967296
./bin 16 4  20.95s user 0.11s system 99% cpu 21.078 total
 ==> 205,010,372 objects per second

% time ./bin 10 8
 # = 1073741824
./bin 10 8  4.35s user 0.02s system 99% cpu 4.376 total
 ==> 246,837,200 objects per second

% time ./bin 8 16
 # = 4294967296
./bin 8 16  18.57s user 0.11s system 99% cpu 18.709 total
 ==> 231,285,260 objects per second
*/



/*
 Timing: (comparing mixedradix_gray with mixedradix_gray2)

---- factorial base:
 time ./bin  // class mixedradix_gray (N==11)
 ct=479001600 == 12!
./bin  2.63s user 0.02s system 99% cpu 2.656 total
12!/2.656 == 180,346,987 numbers per second

 time ./bin  // class mixedradix_gray2 (N==11)
 ct=479001600
./bin  2.20s user 0.02s system 100% cpu 2.217 total
12!/2.217 == 216,058,457 numbers per second


---- binary:
 time ./bin  // class mixedradix_gray (N==31)
 ct=2147483648
./bin  13.94s user 0.06s system 99% cpu 14.005 total
2147483648/14.005 == 153,336,925 numbers per second

 time ./bin  // class mixedradix_gray2 (N==31)
 ct=2147483648
./bin  17.01s user 0.08s system 99% cpu 17.098 total
2147483648/17.098 == 125,598,528 numbers per second

*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-gray2-demo.cc"
/// End:

