
#include "comb/mixedradix-gslex.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"
#include "fxtiomanip.h"

//% Mixed radix numbers in (generalized, subset-) lexicographic order.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits")

    ALLOCA(ulong, r, n);
    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits");
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for second...last digit")
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-2] = atol(argv[k]);

    mixedradix_gslex M(r, n);
//    M.last();

    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();
        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " # = " << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 30 2  ## binary is worst case
 # = 1073741824
./bin 30 2  8.98s user 0.05s system 99% cpu 9.041 total
 ==> 119,570,359 objects per second

% time ./bin 16 4
 # = 4294967296
./bin 16 4  23.33s user 0.12s system 97% cpu 23.990 total
 ==> 184,096,326 objects per second

% time ./bin 10 8
 # = 1073741824
./bin 10 8  4.74s user 0.04s system 99% cpu 4.783 total
 ==> 226,527,810 objects per second

% time ./bin 8 16
 # = 4294967296
./bin 8 16  13.76s user 0.06s system 99% cpu 13.902 total
 ==> 312,134,251 objects per second

*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-gslex-demo.cc"
/// End:

