
#include "comb/mixedradix-modular-gray.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"
#include "fxtiomanip.h"

//% Modular mixed radix Gray code.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits")

    ALLOCA(ulong, r, n);
    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits");
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for second...last digit")
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-2] = atol(argv[k]);

    mixedradix_modular_gray M(r, n);
//    M.last();

    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();
        cout << setw(6) << M.pos();
        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " # = " << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 30 2  ## binary is worst case
 # = 1073741824
./bin 30 2  17.50s user 0.09s system 99% cpu 17.626 total
 ==> 61,356,675 objects per second

% time ./bin 16 4
 # = 4294967296
./bin 16 4  52.50s user 0.28s system 99% cpu 52.827 total
 ==> 81,808,900 objects per second

% time ./bin 10 8
 # = 1073741824
./bin 10 8  11.79s user 0.07s system 99% cpu 11.858 total
 ==> 91,072,249 objects per second

% time ./bin 8 16
 # = 4294967296
./bin 8 16  42.34s user 0.24s system 99% cpu 42.589 total
 ==> 101,439,945 objects per second
*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-modular-gray-demo.cc"
/// End:

