
#include "comb/permgray.h"

#include "comb/permgray-ffact.h"

#include "comb/printperm.h"
#include "comb/printmixedradix.h"

#include "fxttypes.h"
#include "fxtio.h"
#include "demo/nextarg.h"


//% Generate all permutations in minimal-change order.

//#define TIMING // uncomment to disable printing

#define USE_VER1  // uncomment to use class perm_gray (else use perm_gray_ffact)


int
main(int argc, char **argv)
{
    ulong n = 4;
    NXARG(n, "Permutations of n elements.");
    bool dfz = true;  // whether to print dots for zeros

#ifdef USE_VER1
    perm_gray P(n);
#else
    perm_gray_ffact P(n);
#endif

#ifdef TIMING
    while ( P.next() ) {;}
#else
    const ulong *x = P.data();
    const ulong *ix = P.invdata();
    ulong ct = 0;
    ulong sw1, sw2;
    do
    {
        cout << setw(4) << ct << ":";
        ++ct;

        print_perm("   x= ", x, n, dfz);

        P.get_swap(sw1, sw2);
        cout << "  sw(" << sw1 << ", " << sw2 << ")";


#ifdef USE_VER1
        const ulong *a = P.mrg_->data();   // perm_gray
#else
        const ulong *a = P.d_;  // perm_gray_ffact
#endif

        print_mixedradix("   mr= ", a, n-1, dfz);

#ifdef USE_VER1
        cout << "   p=" << P.mrg_->pos();
        cout << "   d=" << (P.mrg_->dir()>0?'+':'-') << 1;
#else
#endif

        print_perm("   ix= ", ix, n, dfz);

        cout << endl;
    }
    while ( P.next() );
#endif // TIMING

    return 0;
}
// -------------------------


/*
 Timing:

 time ./bin 12  // class perm_gray
./bin 12  7.36s user 0.04s system 100% cpu 7.395 total
 12!/7.395 == 64,773,711 permutations per second

 time ./bin 12  // class perm_gray_ffact
./bin 12  5.50s user 0.02s system 99% cpu 5.519 total
12!/5.519 == 86,791,375 permutations per second
*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/permgray-demo.cc"
/// End:

