
#include "comb/subset-gray-delta.h"

#include "comb/printset.h"  // print_deltaset_as_set()
#include "demo/nextarg.h"
#include "fxttypes.h"

#include "fxtio.h"
#include "fxtiomanip.h"


//% Generate all subsets (as delta sets) in minimal-change order.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 5;
    NXARG(n, "Size of the set");
    bool rq = 0;
    NXARG(rq, "Whether to generate subsets in reversed order");

    subset_gray_delta sg(n);
    const ulong *x = sg.data();

#ifdef TIMING
    while ( sg.next() )  {;}

#else // TIMING
    ulong num;
    do
    {
        num = sg.num();
        cout << setw(2) << sg.current() << ":  ";

        // print as bit set:
        for (ulong k=0; k<n; ++k)  cout << (x[k]?'1':'.');
        cout << "   chg @ " << sg.ch_;
        cout << "   #=" << num;

        print_deltaset_as_set("   set=", x, n);
        cout << endl;

        num = ( rq ? sg.prev() : sg.next() );
    }
    while ( num );
#endif // TIMING

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 30
arg 1: 30 == n  [Size of the set]  default=5
arg 2: 0 == rq  [Whether to generate subsets in reversed order]  default=0
./bin 30  9.78s user 0.04s system 100% cpu 9.819 total
 ==> 2^30/9.819 == 109,353,480 subsets per second
*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/subset-gray-delta-demo.cc"
/// End:

